/*
 * $Id: CDialog.h,v 1.2 1998/11/24 06:20:35 mac Exp mac $
 */

#ifndef __CDIALOG_H__
#define __CDIALOG_H__

#include <stl.h>
#include "geometry.h"

#include "CFont.h"
#include "CScrnBuf.h"
#include "CString.h"
#include "types.h"

// forward reference
class CDialog;


class CDialogWidget {
 public:
  CDialogWidget( CDialog* p, int x, int y ): pdlg(p), iX(x), iY(y) {}
  virtual ~CDialogWidget() {
    pdlg = NULL;			// just in case
  }

  int x()			{ return iX; }
  int y()			{ return iY; }
  CDialog* dlg()	{ return pdlg; }

  virtual void paint() {
    // don't want this to be abstract class as it makes trouble in
    // list.h
  }

  virtual void focused()	{}

 protected:
  CDialog* pdlg;
  int iX, iY;
};


class CDlgText: public CDialogWidget {
 public:

  CDlgText( CDialog* p, int x, int y ): CDialogWidget(p,x,y), str(NULL) {
  }
  CDlgText( CDialog* p, int x, int y, const char* sz): CDialogWidget(p,x,y), 
    str((const char *)sz) {
  }

  virtual void paint();
  const char* text()		{ return (const char*)str; }
  void text( const char* sz )	{ str.strdup(sz); }

 protected:
  CString str;
};

class CDlgEdit: public CDlgText {
 public:
	
  CDlgEdit( CDialog* p, int x, int y, int w ): CDlgText(p,x,y), iWidth(w),
    cursorPos(-1) {
  }

  virtual void paint();
  void paintWithCursor();

  int width()	 					{ return iWidth; }

  virtual void focused();

 private:
  int iWidth;
  char cursorPos;
};


class CDialog {
 public:
  CDialog( CScrnBuf* p, int x, int y, int width, int height );
  ~CDialog();

  void paint();
  void show();
  void hide();

  CScrnBuf* getpsb()		{ return psb; }

  void addWidget( CDialogWidget* pwdgt ) {
    widgets.push_back( pwdgt );
  }

  int col()					{ return iColor; }
  int col(int i)				{ return iColor=i; }
  int colBorder()				{ return iBorderColor; }
  int colBorder(int i)		{ return iBorderColor=i; }
  int colText()				{ return iTextColor; }
  int colText(int i)			{ return iTextColor=i; }
  int colEdit()				{ return iEditColor; }
  int colEdit(int i)			{ return iEditColor=i; }
  int colEditBack()			{ return iEditBackColor; }
  int colEditBack(int i)		{ return iEditBackColor=i; }

  int x()						{ return iX; }
  int y()						{ return iY; }

 private:
  CScrnBuf* psb;

  int iX, iY, iWidth, iHeight;
  char* abBackSave;
  bool fVisible;

  int iColor;
  int iBorderColor;
  int iTextColor;
  int iEditColor;
  int iEditBackColor;

  list<CDialogWidget*> widgets;
};


// global declarations
extern CFont* pdlgfnt;

#endif	// __CDIALOG_H__

