/*
 * a superclass from CScrnBuf that is specifically designed to grab
 * pics from a CScrnBuf
 *
 * $Id: CGrabBuf.cc,v 1.3 1998/11/24 06:20:35 mac Exp mac $
 */

#include "CGrabBuf.h"


////////////////////////////// CODE

/*
 * Starting at pnt, scanning first across, then vertically, finds a 
 * non-zero pixel; then scans down until that (left) wall finishes; then
 * scans right to find the end of the (bottom) wall.
 *
 * The returned rectangle includes the bounding box walls
 */
bool 
CGrabBuf::findPic( uvec_t pnt, urect_t& rct ) {

  char* buf = cbuf();

  if( !buf )
    return false;

  while( pnt.y<height() ) {

    if( *(buf + pnt.y*width() + pnt.x) ) {	// found starting corner

      // save starting corner
      uvec_t p1 = pnt;

      while( *(buf + (pnt.y+1)*width() + pnt.x) )
	pnt.y++;

      while( *(buf + pnt.y*width() + pnt.x + 1) )
	pnt.x++;

      // save second corner
      uvec_t p2 = pnt;

      // now save these in the rect
      rct = urect_t( p1, p2 );
      
      return true;
    }

    pnt.x++;
    if( pnt.x >= width() ) {
      pnt.x = 0;
      pnt.y++;
    }
  }

  return false;
}


/*
 * given a starting point 'pnt' and a maximum length 'length', this
 * routine scans that segment (starting at pnt, and ending 'length'
 * units to the right) for any non-zero pixels.
 *
 * returns -1 if none found; else returns the x coordinate of the first
 * non-zero pixel to the right of original 'pnt'
 */
short 
CGrabBuf::scanX( uvec_t pnt, ushort length ) {

  while( length ) {
    if( *(cbuf() + pnt.y*width() + pnt.x) )
      return (short)pnt.x;
    pnt.x++;
    length--;
  }

  return -1;
}

/*
 * given a starting point 'pnt' and a maximum length 'length', this
 * routine scans that segment (starting at pnt, and ending 'length'
 * units below) for any non-zero pixels.
 *
 * returns -1 if none found; else returns the y coordinate of the first
 * non-zero pixel below original 'pnt'
 */
short 
CGrabBuf::scanY( uvec_t pnt, ushort length ) {

  while( length ) {
    if( *(cbuf() + pnt.y*width() + pnt.x) )
      return (short)pnt.y;
    pnt.y++;
    length--;
  }

  return -1;
}


/*
 * collapses the rectangle 'rct' until all of its walls return
 * non-negative scan?s()
 */
void 
CGrabBuf::collapseRect( urect_t& rct ) {

  // do top bound
  while( scanX( rct.p_min(), rct.width() ) < 0 )
    rct.shrink_top();

  // do left bound
  while( scanY( rct.p_min(), rct.height() ) < 0 )
    rct.shrink_left();

  // do bottom bound
  while( scanX( uvec_t(rct.p_min().x, rct.p_max().y), rct.width() ) < 0 )
    rct.shrink_bottom();

  // do right bound
  while( scanY( uvec_t(rct.p_max().x, rct.p_min().y), rct.height() ) < 0 )
    rct.shrink_right();
}


/*
 * extracts a given rectangle into a char* buffer (dynamically
 * allocated)
 *
 * user must deallocate this buffer when done
 */
char* 
CGrabBuf::extractRect( const urect_t& r ) {

  char* res = new char[ (r.width()) * (r.height()) ];
  if( !res )
    return NULL;

  for( ushort i=0;i<r.height(); i++ ) {
    memcpy( res + i*r.width(), 
	    cbuf() + (r.p_min().y+i)*width() + r.p_min().x, 
	    r.width() );
  }

  return res;
}

