/*
 * CScrnBuf.cc
 *
 * Code for the CScrnBuf class.
 * This class represents a screen buffer (a working version of what is
 * displayed on the screen). It consists of the color buffer (the
 * colors of each pixel), as well as a z-buffer (the distance to pixel
 * from player's eye).
 *
 * STATUS:	almost done
 * TO DO:	- put in z-value handling in paint_tile()
 *
 * $Id: CScrnBuf.cc,v 1.6 1998/11/12 01:55:13 mac Exp $
 */

#include <iostream.h>

#include "CScrnBuf.h"
#include "CTile.h"


//#define DEBUG_INFO


const short DZ_HEIGHT=64;
const short DZ_WIDTH=32;
const short DZ_X_HS = 16;
const short DZ_Y_HS = 32;
ushort deltaZ[DZ_HEIGHT][DZ_WIDTH];

void
set_z_map( int x, int y, short val ) {
  if( x<0 || y<0 || x>=DZ_WIDTH || y>=DZ_HEIGHT )
    return;
  deltaZ[y][x] = val;
}

/*
  a CCS pel is from HS down and right
  */
void
make_z_map() {

  short val=0,valStart=1;
  for( int x=DZ_X_HS+1; x>=0; x-=2, val++ ) {
    for( int y=DZ_Y_HS-valStart+1; y<DZ_HEIGHT; y++ ) {
      set_z_map( x, y, val );
      set_z_map( x-1, y, val );
    }
    short val2 = valStart;
    for( int y=DZ_Y_HS-1-valStart+1; y>=0; y-=2, val2++ ) {
      set_z_map( x, y, val2 );
      set_z_map( x-1, y, val2 );
      set_z_map( x, y-1, val2 );
      set_z_map( x-1, y-1, val2 );
    }
    valStart++;
  }

  val=1;
  valStart=1;
  for( int x=DZ_X_HS+2; x<DZ_WIDTH; x+=2, val++ ) {
    for( int y=DZ_Y_HS-valStart+1+1; y<DZ_HEIGHT; y++ ) {
      set_z_map( x, y, val );
      set_z_map( x+1, y, val );
    }
    short val2 = valStart;
    for( int y=DZ_Y_HS-1-valStart+1+1; y>=0; y-=2, val2++ ) {
      set_z_map( x, y, val2 );
      set_z_map( x+1, y, val2 );
      set_z_map( x, y-1, val2 );
      set_z_map( x+1, y-1, val2 );
    }
    valStart++;
  }
}

void
print_z_map() {
  cout << "Dumping z-map" << endl;

  for( int y=0; y<DZ_HEIGHT; y++ ) {
    for( int x=0; x<DZ_WIDTH; x++ ) {
      if( x==DZ_X_HS && y==DZ_Y_HS )
	cout << "(";
      else if( x==DZ_X_HS+1 && y==DZ_Y_HS )
	cout << ")";
      else
	cout << " ";

      if( deltaZ[y][x] < 10 )
	cout << " ";

      cout << deltaZ[y][x];
    }
    cout << endl;
  }
}


/////////////////////////
//// CScrnBuf 

CScrnBuf::CScrnBuf( ushort w, ushort h ): urect_t(w,h) {
  ColBuf = new char[ area() ];
  ZBuf = new depth_t[ area() ];

  ClearCol();
  ClearZbuf();
}


/*
 * makes a copy in this, of the passed CScrnBuf object
 */
bool
CScrnBuf::Copy( const CScrnBuf& sb ) {

  delete ColBuf;
  delete ZBuf;

  // resize
  p_max( sb.p_max() );
  
  ColBuf = new char[ area() ];
  ZBuf = new depth_t[ area() ];

  if( !ColBuf || !ZBuf )
    return false;

  memcpy( ColBuf, sb.ColBuf, area() );
  memcpy( ZBuf, sb.ZBuf, area()*sizeof(depth_t) );

  return true;
};

void 
CScrnBuf::colorize( const urect_t& rct, uchar byColor ) {
  ////// TO DO: make sure we are using the right corner
  char* pchStart = ColBuf + rct.p_min().y*width() + rct.p_min().x;
  for( ushort i=0; i<rct.height()+1; i++ ) {
    memset( pchStart, byColor, rct.width()+1 );
    pchStart += width();
  }
}

/*
  Here, the (x,y) specifies the LL corner of the location where
  the tile is to be painted.

  Although internally the buffer is in UL CS,
  (x,y) here is given in LL coord sys
  */
bool
CScrnBuf::paint_tile( CTile* ptl, short x, short  y, depth_t z ) {

  if( !ptl ) {
    cerr << "CScrnBuf::paintPic(): NULL tile passed" << endl;
    return false;
  }

  short realY = (height() - y -1) - ptl->img_height()+1;

#ifdef DEBUG_INFO
  cout << ">painting " << (char*)ptl->name() << endl;
  cout << "   at x,y == " << x << ", " << realY << "   (ULCS of UL corner)" 
    	<< endl;
#endif

  if( x<0 || realY<0 || x+ptl->img_width()>width() ||
      realY+ptl->img_height()>height() ) {
    cerr << "CScrnBuf::paint_tile(): clipping not yet implemented" << endl;
    return false;
  }

  // otherwise, this is the straight-forward case, without clipping
  char* pbuf = ColBuf + realY*width() + x;
  depth_t* zbuf = ZBuf + realY*width() + x;
  char* pchImg = ptl->img();
  for( short curY=0; curY < ptl->img_height(); curY++ ) {
    for( short curX=0; curX < ptl->img_width(); curX++ ) {
      if( *pchImg ) {
	depth_t zPixel = z + deltaZ
	  [DZ_Y_HS - ((ptl->img_height()-curY-1)-ptl->img_hsy())]
	  [DZ_X_HS + (curX-ptl->img_hsx())];

	/////// puts a red dot on hotspot of each tile
	*(ColBuf + ptl->img_hsx()+x +
	     (ptl->img_height()-1-ptl->img_hsy()+realY) * width()) = 12;

	if( zPixel < *zbuf ) {
	  *pbuf = *pchImg;
	  *zbuf = zPixel;
	}
      }
      pbuf++;
      zbuf++;
      pchImg++;
    }
    pbuf+= width() - ptl->img_width();
    zbuf+= width() - ptl->img_width();
  }

  return true;
}

/*
  x,y is here in LL CS
  */
bool
CScrnBuf::paint_pixel( unsigned x, unsigned  y, uchar col ) {
  if( !encloses( uvec_t(x,y) ) )
    return false;

  y = height() - y -1;
  
  *(ColBuf + y*width() + x) = col;
  return true;
}
