/*
 * $Id: CTile.cc,v 1.9 1998/11/12 01:55:13 mac Exp $
 */

#include <iostream.h>
#include <stdio.h>
#include <stl.h>

#include "CTile.h"
#include "list_aux.h"

#define DEBUG_INFO


const char* szID_Tile="Tile            ";
const char* szTilePath="data/tiles";

struct IndexPair {
  uchar idxOrig;
  uchar idxNew;

  IndexPair( void ): idxOrig(0), idxNew(0) {}
  IndexPair( uchar o, uchar n ): idxOrig(o), idxNew(n) {}
};

bool saveTile( char* buf, ushort w, ushort h, ushort hsx, ushort hsy,
    uchar dx, uchar dy, uchar dz, CPal* ppal, char* szFname ) {

  if( !szFname || !buf )
    return false;


  // what to do if no palette ??? or the other parameters are
  // invalid???

  char* szFullname = new char[ strlen(szFname) + strlen(szTilePath) + 2];
  if( !szFullname )
    return false;
  strcpy( szFullname, szTilePath );
  strcat( szFullname, "/" );
  strcat( szFullname, szFname );

  FILE* fl=fopen( szFullname, "wb" );
  if( !fl )
    return false;

#ifdef DEBUG_INFO
  cout << ">saving tile " << szFullname << endl;
#endif

  fwrite( szID_Tile, strlen(szID_Tile), 1, fl );

  fwrite( &bySecID_image, 1, 1, fl );
  ushort length=w*h + 2*sizeof(ushort);
  fwrite( &length, 2, 1, fl );
  fwrite( &w, 2, 1, fl );
  fwrite( &h, 2, 1, fl );
  fwrite( &hsx, 2, 1, fl );
  fwrite( &hsy, 2, 1, fl );
  fwrite( buf, w*h, 1, fl );

#ifdef DEBUG_INFO
  cout << "   " << "w, h, hsx, hsy == " << w << ", " << h << ", " << hsx
    	<< ", " << hsy << endl;
#endif

  fwrite( &bySecID_dims, 1, 1, fl );
  length=3*sizeof(uchar);
  fwrite( &length, 2, 1, fl );
  fwrite( &dx, 1, 1, fl );
  fwrite( &dy, 1, 1, fl );
  fwrite( &dz, 1, 1, fl );

#ifdef DEBUG_INFO
  cout << "   " << "dx, dy, dz == " << (ushort)dx << ", " << (ushort)dy 
    	<< ", " << (ushort)dz << endl;
#endif

  fwrite( &bySecID_pal, 1, 1, fl );
  length = 0;
  fwrite( &length, 2, 1, fl );
  long lPalStart = ftell( fl );

  ushort cnt=0;
  for( ushort idx=0; idx<256; idx++ ) 
    for( ushort z=0; z<w*h; z++ )
      if( *(buf+z) == ((char)idx) ) {
	cnt++;

	// save this palette entry
	CRGB rgb = ppal->color(idx);
	uchar i= (uchar) idx;
	fwrite( &i, 1, 1, fl );
	fwrite( &rgb.byRed, 1, 1, fl );
	fwrite( &rgb.byGreen, 1, 1, fl );
	fwrite( &rgb.byBlue, 1, 1, fl );
	break;
      }

  length = ftell(fl) - lPalStart;
  fseek( fl, lPalStart-2, SEEK_SET );
  fwrite( &length, 2, 1, fl );

#ifdef DEBUG_INFO
  cout << "   " << cnt << " palette entries saved." << endl;
#endif

  fclose( fl );

  return true;
}


bool 
CTile::load( const char* szFname ) {

  if( !szFname )
    return false;

  char* szFullname = new char[ strlen(szFname) + strlen(szTilePath) + 2];
  if( !szFullname )
    return false;
  strcpy( szFullname, szTilePath );
  strcat( szFullname, "/" );
  strcat( szFullname, szFname );

  FILE* fl;
  if( !(fl=fopen( szFullname, "rb" ) ) )
    return false;

  cout << "loading tile " << szFullname << endl;

  // store the filename of the tile
  strName.strdup( szFname );

  char buf[128];
  fread( buf, 16, 1, fl );
  if( strncmp( buf, szID_Tile, strlen(szID_Tile) ) ) {
    cerr << "CTile::load: " << szFullname << " is not a Tile!" << endl;
    fclose( fl );
    return false;
  }

  for(;;) {
    uchar bySecID;
    ushort length;

    fread( &bySecID, 1, 1, fl );
    if( feof( fl ) )
      break;

    fread( &length, 2, 1, fl );

    switch( bySecID ) {
      case bySecID_image:
	fread( &w, 2, 1, fl );
	fread( &h, 2, 1, fl );
	fread( &hsx, 2, 1, fl );
	fread( &hsy, 2, 1, fl );
	pbyImg = new char[w*h];
	if( !pbyImg ) {
	  cerr << "CTile::load() out of mem" << endl;
	  return false;
	}
	fread( pbyImg, w*h, 1, fl );
	break;
      case bySecID_dims:
	fread( &dx, 1, 1, fl );
	fread( &dy, 1, 1, fl );
	fread( &dz, 1, 1, fl );
	break;
      case bySecID_pal:
	{
	  if( !pbyImg ) {
	    cerr << "CTile::load(): palette given before image! (file=="
	      << szFname << ")" << endl;
	    fclose( fl );
	    return false;
	  }

	  list<IndexPair> lst;

	  for( ushort i=0; i<(length>>2); i++ ) {
	    uchar idxOriginal;
	    CRGB rgb;
	    fread( &idxOriginal, 1, 1, fl );
	    fread( &rgb.byRed, 1, 1, fl );
	    fread( &rgb.byGreen, 1, 1, fl );
	    fread( &rgb.byBlue, 1, 1, fl );
	    short idxNew = gPal.assign( rgb );
	    if( idxNew < 0 ) {
	      cerr << "CTile::load(): palette out of colours!"
		<< endl;
	      fclose( fl );
	      return false;
	    }

	    lst.push_back( IndexPair(idxOriginal,idxNew) );
	  }

	  for( ushort z=0; z<w*h; z++ ) {
	    list<IndexPair>::iterator it = lst.begin();
	    while( it != lst.end() ) {
	      if( (*it).idxOrig == ((uchar)*(pbyImg+z)) ) {
		*(pbyImg+z) = (*it).idxNew;
		break;
	      }
	      ++it;
	    }

	    if( it==lst.end() )
	      cerr << "A colour has been missed in allocation" << endl;
	  }

	  EMPTY( lst );
	}
	break;
      default:
	fseek( fl, length, SEEK_CUR );
	break;
    }
  }

  fclose( fl );

  return true;
}

