/*
 * CWorld.cc
 *
 * $Id: CWorld.cc,v 1.6 1998/11/12 01:55:13 mac Exp $
 */

#include <iostream.h>
#include <fstream.h>
#include <stdio.h>
#include <string.h>

#include "CWorld.h"
#include "CTileMgr.h"

#include <unistd.h>


#define COMMENT_CHAR '#'

// some external stuff
extern CTileMgr gTileMgr;

//////////////////////////////
// CWorld

void
GetRandomName( char* sz ) {
  sprintf( sz, "T%ld.tmp", random() );
}


/*
 * loads a world stored in text file format, with comments
 */

bool
CWorld::textLoadCommented( char* szFilename ) {
  fstream fin( szFilename, ios::in );

  if( !fin ) {
    cerr << "CWorld::textLoadCommented(): file not found" << endl;
    return false;
  }

  char szTmpFname[128];
  for(;;) {

    GetRandomName( szTmpFname );	// should be really using temp area
    fstream ftst( szTmpFname, ios::in );
    if( !ftst )
      break;
  }

  fstream fout( szTmpFname, ios::out );
  if( !fout ) {
    cerr << "CWorld::textLoadCommented(): unable to open temp file" << endl;
    return false;
  }

  char c;
  bool bComment = false;
  while( fin.get(c) ) {

    if( bComment ) {
      if( c == '\n' )
	bComment = false;
    }
    else {
      if( c == COMMENT_CHAR )
	bComment = true;
      else
	fout.put(c);
    }
  }

  fout.close();
  bool bRes = textLoad( szTmpFname );

  // now delete the temp file
  unlink( szTmpFname );
	
  return bRes;
}


bool
CWorld::textLoad( char* szFilename ) {
  fstream fin( szFilename, ios::in );
  if( !fin ) {
    cerr << "CWorld::textLoad(): file not found:" << szFilename << endl;
    return false;
  }

  char buf[128];
  fin.getline( buf, 128 );
  szName = new char[strlen(buf)+1];
  if( !szName ) {
    cerr << "CWorld::textLoad(): out of mem" << endl;
    return false;
  }
  strcpy( szName, buf );
  cout << "Loading world: " << szName << endl;

  unsigned ww, wd;
  fin >> ww >> wd;

  if( ww==0 || wd==0 ) {
    cerr << "CWorld::textLoad(): invalid world dimensions ("
	 << ww << "," << wd << ")" << endl;
    return false;
  } else {
    urect_t::p_min( uvec_t(0,0) );
    urect_t::p_max( uvec_t(ww-1,wd-1) );
  }

  make_pacols();
//   paCols = new CColumn[ area() ];
//   if( !paCols ) {
//     cerr << "CWorld::textLoad(): could not allocate the column array" << endl;
//     return false;
//   }

  ushort x, y, z, cell_x, cell_y;
  while( !fin.eof() ) {
    fin >> x >> y >> z >> buf >> cell_x >> cell_y;
    if( fin.eof() )
      continue;
    
    CCell* pcel = find_or_make_cell( x, y, z );
    if( !pcel ) {
      cerr << "CWorld::textLoad(): unable to find or make a cell" << endl;
      return false;
    }

    CTile* ptl = gTileMgr.get_tile( buf );
    if( !ptl ) {
      cerr << "CWorld::textLoad(): unable to find or load a tile"
	   << " (" << buf << ")" << endl;
      return false;
    }

    CObject* pobj = new CObject;
    if( !pobj ) {
      cerr << "CWorld::textLoad(): out of memory" << endl;
      return false;
    }

    pobj->tile( ptl );
    pobj->cell_x( cell_x );
    pobj->cell_y( cell_y );
      
    pcel->add_obj( pobj );
  }
    
  return true;
}

CCell*
CWorld::find_or_make_cell( ushort x, ushort y, short z ) {

  if( x >= width() || y >= depth() ) {
    //    cerr << "CWorld::find_or_make_cell(): invalid coordinate" << endl;
    return NULL;
  }

  CColumn* pcol = find_col( x, y );
  if( !pcol ) {
    cerr << "CWorld::find_or_make_cell(): column is missing" << endl;
    return NULL;
  }

  return pcol->find_or_make_cell( z );
} 
  
CColumn*
CWorld::find_col( ushort x, ushort y ) const {
  if( x >= width() || y >= depth() ) {
    //    cerr << "CWorld::find_col(): invalid coords" << endl;
    return NULL;
  }

  return (paCols + y*width() + x );
}


ivec_t
CWorld::wcs( const ivec_t& wccs, const ivec_t& ccs ) const {

  ivec_t res;
  res.x = wccs.x*cellXYsize + ccs.x;
  res.y = wccs.y*cellXYsize + ccs.y;
  res.z = wccs.z*cellZsize + ccs.z;

  return res;
}

void
CWorld::make_pacols( void ) {
  paCols = new CColumn[ area() ];
}
