/*
 * CWorld.h
 *
 * header file for the CWorld class
 *
 * $Id: CWorld.h,v 1.5 1998/11/12 01:55:13 mac Exp $
 */

#ifndef __CWORLD_H__
#define __CWORLD_H__

#include "geometry.h"
#include "types.h"

#include "CColumn.h"
/* #include "CCell.h" */

////////////////////////////////
// forward refs
class CColumn;
class CCell;

class CWorld: public urect_t {
  char*		szName;
  CColumn*	paCols;

  uchar		cellXYsize;
  uchar		cellZsize;
  
public:
  CWorld( uchar cw, uchar ch ):
    urect_t(0,0), szName(NULL), paCols(NULL), cellXYsize(cw),
    cellZsize(ch) {}
  CWorld( uchar cw, uchar ch, char* szFilename ):
    urect_t(0,0), cellXYsize(cw), cellZsize(ch)
    { textLoadCommented( szFilename ); }
  ~CWorld() {
    delete [] paCols;
    delete szName;
    paCols = NULL;
  }
	
  bool textLoadCommented( char* szFilename );
  bool textLoad( char* szFilename );

  CColumn* cols() const { return paCols; }
  const char* name() const { return szName; }
  unsigned depth() const { return height(); }

  // here below, z is specified in terms of cells, not altitude
  CCell* find_or_make_cell( ushort x, ushort y, short z );

  CColumn* find_col( ushort x, ushort y ) const;

  uchar cell_xy_width() const { return cellXYsize; }
  uchar cell_z_height() const { return cellZsize; }

  ivec_t wcs( const ivec_t& wccs, const ivec_t& ccs ) const;

 private:
  void make_pacols( void );
};

#endif // __CWORLD_H__















