/*
 * grabs fonts from a .PCX file
 *
 * $Id: fontgrab.cc,v 1.1 1997/10/30 05:38:56 maciek Exp $
 *
 * $Log: fontgrab.cc,v $
 * Revision 1.1  1997/10/30 05:38:56  maciek
 * Initial revision
 *
 * Revision 1.2  1997/08/20 18:25:04  maciek
 * CFont basic work finished
 *
 *
 */

#include <iostream.h>
#include <stdio.h>

#include "CGrabBuf.h"
#include "CFont.h"
#include "gfx.h"
#include <pcx.h>


const width=320;
const height=240;

#define FNAME_ARG	1

/////////////////////////////
// main
//
int main( int argc, char* argv[] ) {

	gfx_droproot();

	if( argc != FNAME_ARG+1 ) {
		cerr << "error: need to specify a .pcx file from which to grab the font"
			<< endl;
		return -1;
	}
		
	CGrabBuf gb( width, height );
	CPal pal;

	if( !loadPCXpalette( argv[FNAME_ARG], pal ) ) {
		cerr << "error: unable to load palette from file " 
			 << argv[FNAME_ARG] << endl;
		return -2;
	}

	// erase buffer just in case the PCX is too short
	memset( gb.cbuf(), 0, width*height );

	if( !loadPCXdata( argv[FNAME_ARG], gb.cbuf(), width, height ) ) {
		cerr << "error: unable to get picture data from file "
			 << argv[FNAME_ARG] << endl;
		return -3;
	}


	FILE* fl = fopen( "output.fnt", "wb" );
	if( !fl ) {
		cerr << "error: unable to open file for output" << endl;
		return -4;
	}

	fontFileHeader ffh;
	strcpy( ffh.id, "Font file       " );
	ffh.majVer = 0;
	ffh.minVer = 1;
	ffh.beginDataOffset = sizeof(ffh);
	ffh.baseHeight = 0;
	ffh.stemHeight = 0;

	// write the header
	fwrite( &ffh, 1, sizeof(ffh), fl );

	gfx_on();

	pal.install();

	// now find all the pics
	Vector2<ushort> pntStart(0,0);
	for(;;) {

		gb.BlitToScrn();

		Rect<ushort> rct,rctLtr,rctBound;
		if( !gb.findPic( pntStart, rct ) )
			break;					// no more pics

		rctLtr=rctBound=rct;
		rctLtr.shrinkAll();			// exclude the bounding box
		rct.shrinkAll(-1);			// include the marker on left

		short tick = gb.scanY( rct.p1, rct.height() );
		if( tick == -1 ) {
			cerr << "Missing tick" << endl;
			exit(-5);
		}

		char droopsBy = rctBound.p2.y - tick - 1;

		// get the rectangle here
		char* buf = new char[ rctLtr.width() * rctLtr.height() ];
		if( !buf ) {
			cerr << "out of mem" << endl;
			exit(-6);
		}
		gfx_getbox( rctLtr.p1.x, rctLtr.p1.y, rctLtr.width(), rctLtr.height(),
											buf );
		// highlight the rect
		gfx_setcolor( 14 );
		gfx_rect( rctBound );
		gfx_pixel( rct.p1.x, tick );
		
		char die=gfx_getch();
		uchar width = rctLtr.width();
		uchar height = rctLtr.height();

		// save the die
		fwrite( &die, sizeof(die), 1, fl );
		fwrite( &droopsBy, sizeof(droopsBy), 1, fl );
		fwrite( &width, 1, 1, fl );
		fwrite( &height, 1, 1, fl );
		fwrite( buf, rctLtr.width()*rctLtr.height(), 1, fl );

		// now wipe it off
		gfx_fillbox( rct.p1.x, rct.p1.y, rct.width(), rct.height(), 0 );

		// figure out whether this die exceeds previous base/stem/Height
		uchar baseHeight, stemHeight;
		baseHeight = height - droopsBy;
		stemHeight = droopsBy;

		if( baseHeight > ffh.baseHeight )
			ffh.baseHeight = baseHeight;
		if( stemHeight > ffh.stemHeight )
			ffh.stemHeight = stemHeight;
	}

	// now update the header
	rewind( fl );
	fwrite( &ffh, sizeof(ffh), 1, fl );

	fclose( fl );

	gfx_off();
}


