/*
 * a superclass from CScrnBuf that is specifically designed to grab
 * pics from a CScrnBuf
 *
 * $Id: CGrabBuf.cc,v 1.1 1997/08/20 18:25:04 maciek Exp $
 *
 * $Log: CGrabBuf.cc,v $
 * Revision 1.1  1997/08/20 18:25:04  maciek
 * CFont basic work finished
 *
 *
 */

#include "CGrabBuf.h"


////////////////////////////// CODE

/*
 * Starting at pnt, scanning first across, then vertically, finds a 
 * non-zero pixel; then scans down until that (left) wall finishes; then
 * scans right to find the end of the (bottom) wall.
 *
 * The returned rectangle includes the bounding box walls
 */
bool 
CGrabBuf::findPic( Vector2<ushort> pnt, Rect<ushort>& rct ) {

	char* buf = cbuf();

	if( !buf )
		return false;

	while( pnt.y<height() ) {

		if( *(buf + pnt.y*width() + pnt.x) ) {	// found starting corner

			// save starting corner
			rct.p1 = pnt;

			while( *(buf + (pnt.y+1)*width() + pnt.x) )
				pnt.y++;

			while( *(buf + pnt.y*width() + pnt.x + 1) )
				pnt.x++;

			// save second corner
			rct.p2 = pnt;

			return true;
		}

		pnt.x++;
		if( pnt.x >= width() ) {
			pnt.x = 0;
			pnt.y++;
		}
	}

	return false;
}


/*
 * given a starting point 'pnt' and a maximum length 'length', this
 * routine scans that segment (starting at pnt, and ending 'length'
 * units to the right) for any non-zero pixels.
 *
 * returns -1 if none found; else returns the x coordinate of the first
 * non-zero pixel to the right of original 'pnt'
 */
short 
CGrabBuf::scanX( Vector2<ushort> pnt, ushort length ) {

	while( length ) {
		if( *(cbuf() + pnt.y*width() + pnt.x) )
			return (short)pnt.x;
		pnt.x++;
		length--;
	}

	return -1;
}

/*
 * given a starting point 'pnt' and a maximum length 'length', this
 * routine scans that segment (starting at pnt, and ending 'length'
 * units below) for any non-zero pixels.
 *
 * returns -1 if none found; else returns the y coordinate of the first
 * non-zero pixel below original 'pnt'
 */
short 
CGrabBuf::scanY( Vector2<ushort> pnt, ushort length ) {

	while( length ) {
		if( *(cbuf() + pnt.y*width() + pnt.x) )
			return (short)pnt.y;
		pnt.y++;
		length--;
	}

	return -1;
}


/*
 * collapses the rectangle 'rct' until all of its walls return
 * non-negative scan?s()
 */
void 
CGrabBuf::collapseRect( Rect<ushort>& rct ) {

	// do top bound
	while( scanX( rct.p1, rct.width() ) < 0 )
		rct.shrinkTop();

	// do left bound
	while( scanY( rct.p1, rct.height() ) < 0 )
		rct.shrinkLeft();

	// do bottom bound
	while( scanX( Vector2<ushort>(rct.p1.x, rct.p2.y), rct.width() ) < 0 )
		rct.shrinkBottom();

	// do right bound
	while( scanY( Vector2<ushort>(rct.p2.x, rct.p1.y), rct.height() ) < 0 )
		rct.shrinkRight();
}


/*
 * extracts a given rectangle into a char* buffer (dynamically
 * allocated)
 *
 * user must deallocate this buffer when done
 */
char* 
CGrabBuf::extractRect( const Rect<ushort>& r ) {

	char* res = new char[ (r.width()+1) * (r.height()+1) ];
	if( !res )
		return NULL;

	for( ushort i=0;i<r.height()+1; i++ ) {
		memcpy( res + i*(r.width()+1), 
				cbuf() + (r.p1.y+i)*width() + r.p1.x, 
				r.width()+1 );
	}

	return res;
}

