/*
 * CScrnBuf.cc
 *
 * Code for the CScrnBuf class.
 * This class represents a screen buffer (a working version of what is
 * displayed on the screen). It consists of the color buffer (the
 * colors of each pixel), as well as a z-buffer (the distance to pixel
 * from player's eye).
 *
 * STATUS:	almost done
 * TO DO:	- put in z-value handling in PaintPic()
 *
 * $Id: CScrnBuf.cc,v 1.7 1997/08/07 17:23:50 maciek Exp $
 *
 * $Log: CScrnBuf.cc,v $
 * Revision 1.7  1997/08/07 17:23:50  maciek
 * put in RCS header info in files
 *
 *
 */

#include "CScrnBuf.h"

/////////////////////////
//// CScrnBuf 

/*
 * constructor
 *
 * status: done
 */
CScrnBuf::CScrnBuf( ushort w, ushort h ) {

	wWidth = w;
	wHeight = h;

	ColBuf = new char[ w*h ];
	ZBuf = new t_DEPTH[ w*h ];

	gfx_setcontextvirtual( wWidth, wHeight, 1, 8, ColBuf );
	pgc = gfx_allocatecontext();
	gfx_getcontext( pgc );
}

/*
 * draws the specified CPic in the screen buffer, using the specified z
 * value, and at the specified coordinates
 *
 * status: only ColBuf implemented; no ZBuf stuff yet.
 */
bool
CScrnBuf::PaintPic( const CPic* pPic, ushort x, ushort y, t_DEPTH zVal ) {

	if( !pPic )
		return false;

	short X,Y;
	X = x + pPic->dx();
	Y = y + pPic->dy();

	if( X<0 )
		X=0;
	if( Y<0 )
		Y=0;
	if( X>=wWidth )
		X=wWidth-1;
	if( Y>=wHeight )
		Y=wHeight-1;

	char* pcCurPic = pPic->data();
	char* pcCurScrn = ColBuf + Y*wWidth + X;

	for( int r=0; r<pPic->height(); r++ ) {
		for( int c=0; c<pPic->width(); c++ ) {

			if( *pcCurPic ) {			// check for opaque pixel in pic
				*pcCurScrn = *pcCurPic;
			}

			pcCurScrn++;
			pcCurPic++;
		}

		pcCurScrn += wWidth - pPic->width();
	}

	return true;
}


/*
 * draws the specified CTile in the screen buffer, using the specified z
 * value, and at the specified coordinates (uses PaintPic())
 */
bool
CScrnBuf::PaintTile( const CTile* pTile, ushort x, ushort y, t_DEPTH zVal ) {

	if( !pTile )
		return false;

	return PaintPic( pTile->pic(), x + pTile->dx(), y + pTile->dy(), zVal );
}



/*
 * blits the screen buffer straight to video memory, which is displayed
 * on screen
 *
 * status: done
 */
void
CScrnBuf::BlitToScrn() {

	// the vgagl way
	gfx_setcontext( pgc );
	gfx_copyscreen( physicalscreen );

/************************* this is the vga (non vgagl) way
	gfx_copytoplanar256( ColBuf, wWidth, 0, wWidth/4, wWidth, wHeight );
 */

}


/*
 * makes a copy in this, of the passed CScrnBuf object
 *
 * status: done
 */
bool
CScrnBuf::Copy( const CScrnBuf& csb ) {

	delete ColBuf;
	delete ZBuf;

	wWidth = csb.wWidth;
	wHeight = csb.wHeight;

	ColBuf = new char[ wWidth*wHeight ];
	ZBuf = new t_DEPTH[ wWidth*wHeight ];

	if( !ColBuf || !ZBuf )
		return false;

	memcpy( ColBuf, csb.ColBuf, wWidth*wHeight );
	memcpy( ZBuf, csb.ZBuf, wWidth*wHeight*sizeof(t_DEPTH) );

	return true;
};

void 
CScrnBuf::colRect( const Rect<ushort>& rct, uchar byColor ) {

	char* pchStart = ColBuf + rct.p1.y*wWidth + rct.p1.x;
	ushort height = rct.height();
	for( ushort i=0; i<height+1; i++ ) {
		memset( pchStart, byColor, rct.width()+1 );
		pchStart += wWidth;
	}
}


