/*
 * load.cc
 *
 * routines for loading a world
 *
 * $Id: load.cc,v 1.5 1997/08/07 17:23:50 maciek Exp $
 *
 * $Log: load.cc,v $
 * Revision 1.5  1997/08/07 17:23:50  maciek
 * put in RCS header info in files
 *
 *
 *
 */

#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>

#include <stdio.h>
#include <stdlib.h>

#include <unistd.h>

#include "CString.h"
#include "CWorld.h"
#include "main.h"

#include <list.h>

#define COMMENT_CHAR	'#'
#define IO_BUF_SIZE		(80)

/////////////////////////////////////////
// local structures
//
class CPicPair {
public:
	CString strID;
	CPic* pPic;

	CPicPair( const char* sz, CPic* p ): strID(sz), pPic(p) {}
};

class CTilePair {
public:
	CString strID;
	CTile* pTile;

	CTilePair( const char* sz, CTile* p ): strID(sz), pTile(p) {}
};


///////////////////////////////////////////
// globals
//
dlist<CPicPair> lstPicPairs;
dlist<CTilePair> lstTilePairs;

////////////////////////////////////////////
// utility functions
//

CPic* 
GetPic( const char* szID ) {

	if( !szID )
		return NULL;

	dlist_iter<CPicPair> it( lstPicPairs );

	while( it++ ) {
		if( !strcmp( it()->strID, szID ) ) 
			return it()->pPic;
	}

	return NULL;
}

CTile*
GetTile( const char* szID ) {

	if( !szID )
		return NULL;

	dlist_iter<CTilePair> it( lstTilePairs );

	while( it++ ) {
		if( !strcmp( it()->strID, szID ) )
			return it()->pTile;
	}

	return NULL;
}


inline
void
GetRandomName( char* sz ) {

	sprintf( sz, "T%ld.tmp", random() );
}


//////////////////////////////////////////
// CWorld code for loading worlds
//

/*
 * loads a world stored in text file format, with comments
 */

bool
CWorld::textLoadCommented( char* szFilename ) {

	fstream fin( szFilename, ios::in );

	if( !fin )
		return false;

	char szTmpFname[128];
	for(;;) {

		GetRandomName( szTmpFname );		// should be really using temp area
		fstream ftst( szTmpFname, ios::in );
		if( !ftst )
			break;
	}

	fstream fout( szTmpFname, ios::out );
	if( !fout )
		return false;

	char c;
	bool bComment = false;
	while( fin.get(c) ) {

		if( bComment ) {
			if( c == '\n' )
				bComment = false;
		}
		else {
			if( c == COMMENT_CHAR )
				bComment = true;
			else
				fout.put(c);
		}
	}

	fout.close();
	bool bRes = textLoad( szTmpFname );

	// now delete the temp file
	unlink( szTmpFname );
	
	return bRes;
}


/*
 * loads a world stored in the text file format
 */
bool
CWorld::textLoad( char* szFilename ) {

	fstream fin( szFilename, ios::in );

	if( !fin ) 
		return false;

	char sz[IO_BUF_SIZE];
	fin.getline( sz, IO_BUF_SIZE );
	szName = new char[strlen(sz) + 1];
	if( !szName )
		return false;
	strcpy( szName, sz );

	// read in the dimensions and make the array of CColumn
	fin >> wWidth >> wDepth;
	paCols = new CColumn[ wWidth*wDepth ];
	if( !paCols )
		return false;

	if( fin.eof() ) 
		return false;

	if( !readTextPics( fin ) )
		return false;

	if( !readTextTiles( fin ) )
		return false;

	if( !readTextCols( fin ) )
		return false;

	// this optional, until we have CObjects...
	if( !readTextObjs( fin ) )
		return false;

	// now just free up some memory
	lstPicPairs.empty();
	lstTilePairs.empty();

	return true;
}

/*
 * reads the IDs and pic names
 */
bool
CWorld::readTextPics( fstream& fin ) {
	
	char szID[IO_BUF_SIZE];
	char szFname[IO_BUF_SIZE];

	for(;;) {

		fin >> setw(IO_BUF_SIZE) >> szID;
		if( !strcmp( szID, "_end_pic_" ) ) 
			break;

		// get the pic filename string
		fin >> setw(IO_BUF_SIZE) >> szFname;
		if( fin.eof() )
			return false;

		// now load the pic and put it in the list of pics
		CPic* pPic = new CPic;	
		lstPics.append( pPic );
		if( !pPic->load( szFname ) )
			return false;

		// keep the pointer to this pic using the index
		lstPicPairs.append( new CPicPair( szID, pPic ) );
	}

	return true;
}



/*
 * reads the IDs and specs of the tiles
 */
bool
CWorld::readTextTiles( fstream& fin ) {

	char szTileID[IO_BUF_SIZE];
	char szPicID[IO_BUF_SIZE];
	short dx, dy;

	for(;;) {

		fin >> setw(IO_BUF_SIZE) >> szTileID;
		if( !strcmp( szTileID, "_end_tile_" ) )
			break;

		// get the pic index
		fin >> setw(IO_BUF_SIZE) >> szPicID;
		fin >> dx >> dy;
		if( fin.eof() )
			return false;

		// create the tile
		CTile* pTile = new CTile( GetPic( szPicID ), dx, dy );
		if( !pTile )
			return false;
		lstTiles.append( pTile );

		// make a note of the pair
		lstTilePairs.append( new CTilePair( szTileID, pTile ) );
	}

	return true;
}


/*
 * reads in the columns using the text file format
 */
bool
CWorld::readTextCols( fstream& fin ) {

	ushort x=0, y=0;			// the current column in the world
	ushort cCells;
	char sz[IO_BUF_SIZE];
	char szLTileID[IO_BUF_SIZE];
	char szRTileID[IO_BUF_SIZE];
	char szBTileID[IO_BUF_SIZE];
	char szFLTileID[IO_BUF_SIZE];
	char szFRTileID[IO_BUF_SIZE];
	char szTTileID[IO_BUF_SIZE];

	for(;;) {
		fin >> setw(IO_BUF_SIZE) >> sz;

		if( !strcmp( sz, "COLXY" ) )
			fin >> x >> y >> cCells;
		else if ( !strcmp( sz, "COL" ) )
			fin >> cCells;
		else if ( !strcmp( sz, "_end_col_" ) )
			break;
		else
			return false;			// error: wrong keyword

		short altitude;
		for( ushort cell=0; cell<cCells; cell++ ) {

			// get each cell, one by one
			fin >> altitude;
			fin >> setw(IO_BUF_SIZE) >> szLTileID;
			fin >> setw(IO_BUF_SIZE) >> szRTileID;
			fin >> setw(IO_BUF_SIZE) >> szBTileID;
			fin >> setw(IO_BUF_SIZE) >> szFLTileID;
			fin >> setw(IO_BUF_SIZE) >> szFRTileID;
			fin >> setw(IO_BUF_SIZE) >> szTTileID;
			CColumn* pCol = paCols + y*wWidth + x;
			CCell* pCell = new CCell( altitude, GetTile( szLTileID ), 
							GetTile( szRTileID ), GetTile( szBTileID ),
							GetTile( szFLTileID ), GetTile( szFRTileID ),
							GetTile( szTTileID ) );
			if( !pCell )
				return false;

			pCol->lstCells.append( pCell );
		}

		x++;
		if( x>=wWidth ) {
			x=0;
			y++;
		}
	}

	return true;
}


/*
 * reads in the objects, using the text file format
 */
bool
CWorld::readTextObjs( fstream& fin ) {

	char sz[IO_BUF_SIZE];

	for(;;) {
		fin >> setw(IO_BUF_SIZE) >> sz;
		if( !strcmp( sz, "_end_obj_" ) )
			break;
		else if( !strcmp( sz, "OBJ" ) )
			fin >> szX >> szY >> szAlt >> szTile;
		else
			return false;

		


}



