/*
 * $Id: pcx.cc,v 1.3 1998/11/24 06:20:35 mac Exp mac $
 */

#include <iostream.h>
#include <stdio.h>			// FILE

#include "pcx.h"

/*
 * loads the pixel data into a buffer from a PCX file named <szFname>
 *
 * the width and height of the buffer are passed to the subroutine so
 * that it can check that the data will not overflow
 */
bool loadPCXdata( const char* szFname, char* buf, ushort w, ushort h ) {

  PcxFileHeader pfh;
  FILE* fl = fopen( szFname, "rb" );

  if( !fl )
    return false;

  fread( &pfh, sizeof(PcxFileHeader), 1, fl );

  if( pfh.bVersion<5 )			// that's the format we know
    return false;
	
  // for now, if the dimensions of picture do not match those of
  // buffer then return an error
  if( ( pfh.uWindowXmax - pfh.uWindowXmin +1 != w ) ||
      ( pfh.uWindowYmax - pfh.uWindowYmin +1 > h ) )
    return false;

  cerr << "loading width == " << pfh.uWindowXmax - pfh.uWindowXmin +1 << endl;
  
  ushort uc;
  char byControl;
  char byData;
  ulong offs=0;
  ulong ulOffsetStop;

  // find out the stopping offset
  fseek( fl, -769, 2 );
  if( fgetc( fl ) != 12 ) {
    fseek( fl, 0, 2 );
    ulOffsetStop = ftell( fl ) + 1;
  }
  else
    ulOffsetStop = ftell( fl );

  fseek( fl, sizeof(pfh), 0 );

  while( !feof( fl ) && ((ulong)ftell( fl ) < ulOffsetStop-1) ) {

    byControl= fgetc( fl );
    if( feof( fl ) )
      break;

    if( byControl&0x80 && byControl&0x40 ) {// if run length
      uc=byControl&0x3f;	// extract the run length
      byData=fgetc( fl );	// get the data byte
      if( feof( fl ) )
	return false;		// premature end of data
    }
    else {
      uc=1;
      byData=byControl;
    }

    for (;uc;uc--) {
      buf[offs]=byData;
      offs++;
    }
  }

  return true;
}


/*
 * loads a vga (256 color) palette from a PCX file
 */
bool loadPCXpalette( const char* szFname, CPal& pal ) {

  FILE* fl = fopen( szFname, "rb" );
  if( !fl )
    return false;

  PcxFileHeader pfh;
  fread(&pfh, sizeof(pfh), 1, fl);  // read in header

  if( pfh.bVersion<5 )		// has to be high enough version
    return false;

  fseek(fl, -769, 2);		// point to start of palette

  if( fgetc( fl ) != 12 )	// should be there to indicate palette
    return false;

  char buf[256*3];
  for( unsigned i=0;i< 256; i++ ) {
    *(buf+i*3) = fgetc( fl ) >> 2;
    *(buf+i*3+1) = fgetc( fl ) >> 2;
    *(buf+i*3+2) = fgetc( fl ) >> 2;
  }

  pal.assignAll( buf );

  return true;
}

