/*
 *
 * $Id: pcx.h,v 1.2 1998/11/24 06:20:35 mac Exp mac $
 *
 * $Log: pcx.h,v $
 * Revision 1.2  1998/11/24 06:20:35  mac
 * chronic check-in
 *
 * Revision 1.1  1997/10/30 05:38:56  maciek
 * Initial revision
 *
 * Revision 1.2  1997/08/07 17:23:50  maciek
 * put in RCS header info in files
 *
 *
 */

#ifndef __PCX_H__
#define __PCX_H__

#include "CPal.h"

/******************************************************************************/
/*                                                                            */
/*  ZSoft .PCX FILE HEADER FORMAT                                             */
/*                                                                            */
/*  Byte    Item            Size    Description/Comments                      */
/*                                                                            */
/*  0       Manufacturer    1       Constant Flag  10 = ZSoft .PCX            */
/*  1       Version         1       Version information:                      */
/*                                  0 = Version 2.5                           */
/*                                  2 = Version 2.8 w/palette information     */
/*                                  3 = Version 2.8 w/o palette information   */
/*                                  5 = Version 3.0                           */
/*  2       Encoding        1       1 = .PCX run length encoding              */
/*  3       Bits per pixel  1       Number of bits/pixel per plane            */
/*  4       Window          8       Picture Dimensions                        */
/*                                  (Xmin, Ymin) - (Xmax - Ymax)              */
/*                                  in pixels, inclusive                      */
/*  12      HRes            2       Horizontal Resolution of creating device  */
/*  14      VRes            2       Vertical Resolution of creating device    */
/*  16      Colormap        48      Color palette setting, see text           */
/*  64      Reserved        1                                                 */
/*  65      NPlanes         1       Number of color planes                    */
/*  66      Bytes per Line  2       Number of bytes per scan line per         */
/*                                  color plane (always even for .PCX files)  */
/*  68      Palette Info    2       How to interpret palette - 1 = color/BW,  */
/*                                  2 = grayscale                             */
/*  70      Filler          58      blank to fill out 128 byte header         */
/*                                                                            */
/*  ~NOTES:                                                                   */
/*  ~All sizes are measured in BYTES.                                         */
/*  All variables of size 2 are integers.                                     */
/*                                                                            */
/******************************************************************************/

struct RGB {
	uchar byRed;
	uchar byGreen;
	uchar byBlue;
};

struct PcxFileHeader {
    uchar         bManufacturer;          // 10==ZSoft .PCX
    uchar         bVersion;               // see above for translation
    uchar         bEncoding;              // 1==.PCX run length encoding
    uchar         bBitsPerPixel;          // number of bits/pixel per plane
    ushort        uWindowXmin;            // Picture Dimensions
    ushort        uWindowYmin;            //         "
    ushort        uWindowXmax;            //         "
    ushort        uWindowYmax;            //         "
    ushort        uHRes;                  // Horizontal Res. of org. device
    ushort        uVRes;                  // Vertical Resolution of org. device
    RGB           rgbPalette1[16];        // palette for first 16 colours
    uchar         bRes1;                  // reserved byte
    uchar         bPlanes;                // number of planes
    ushort        uBytesPerLine;          // bytes per scanline per plane (even)
    ushort        uPaletteInfo;           // ?????
    uchar         bFiller[58];            // filler
};

bool loadPCXdata( const char* szFname, char* buf, ushort w, ushort h );
bool loadPCXpalette( const char* szFname, CPal& pal );

#endif // __PCX_H__
